@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2005-2008. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2005-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -16,15 +16,18 @@
 #ifndef BOOST_INTERPROCESS_SCOPED_LOCK_HPP
 #define BOOST_INTERPROCESS_SCOPED_LOCK_HPP
 
-#if (defined _MSC_VER) && (_MSC_VER >= 1200)
+#if defined(_MSC_VER)
 #  pragma once
 #endif
 
 #include <boost/interprocess/detail/config_begin.hpp>
 #include <boost/interprocess/detail/workaround.hpp>
+#include <boost/interprocess/interprocess_fwd.hpp>
 #include <boost/interprocess/sync/lock_options.hpp>
 #include <boost/interprocess/exceptions.hpp>
-#include <boost/interprocess/detail/move.hpp>
+#include <boost/interprocess/detail/mpl.hpp>
+#include <boost/interprocess/detail/type_traits.hpp>
+#include <boost/move/utility_core.hpp>
 #include <boost/interprocess/detail/posix_time_types_wrk.hpp>
 
 //!\file
@@ -33,11 +36,6 @@
 namespace boost {
 namespace interprocess {
 
-template<class M>
-class sharable_lock;
-
-template<class M>
-class upgradable_lock;
 
 //!scoped_lock is meant to carry out the tasks for locking, unlocking, try-locking
 //!and timed-locking (recursive or not) for the Mutex. The Mutex need not supply all
@@ -52,14 +50,14 @@ class upgradable_lock;
 template <class Mutex>
 class scoped_lock
 {
-   /// @cond
+   #if !defined(BOOST_INTERPROCESS_DOXYGEN_INVOKED)
    private:
    typedef scoped_lock<Mutex> this_type;
-   scoped_lock(scoped_lock const&);
-   scoped_lock& operator=  (scoped_lock const&);
+   BOOST_MOVABLE_BUT_NOT_COPYABLE(scoped_lock)
    typedef bool this_type::*unspecified_bool_type;
-   /// @endcond
+   #endif   //#ifndef BOOST_INTERPROCESS_DOXYGEN_INVOKED
    public:
+
    typedef Mutex mutex_type;
 
    //!Effects: Default constructs a scoped_lock.
@@ -80,18 +78,18 @@ class scoped_lock
    //!Postconditions: owns() == false, and mutex() == &m.
    //!Notes: The constructor will not take ownership of the mutex. There is no effect
    //!   required on the referenced mutex.
-   scoped_lock(mutex_type& m, detail::defer_lock_type)
+   scoped_lock(mutex_type& m, defer_lock_type)
       : mp_mutex(&m), m_locked(false)
    {}
 
    //!Postconditions: owns() == true, and mutex() == &m.
    //!Notes: The constructor will suppose that the mutex is already locked. There
    //!   is no effect required on the referenced mutex.
-   scoped_lock(mutex_type& m, detail::accept_ownership_type)
+   scoped_lock(mutex_type& m, accept_ownership_type)
       : mp_mutex(&m), m_locked(true)
    {}
 
-   //!Effects: m.try_lock(). 
+   //!Effects: m.try_lock().
    //!Postconditions: mutex() == &m. owns() == the return value of the
    //!   m.try_lock() executed within the constructor.
    //!Notes: The constructor will take ownership of the mutex if it can do
@@ -99,11 +97,11 @@ class scoped_lock
    //!   locking depends upon the mutex. If the mutex_type does not support try_lock,
    //!   this constructor will fail at compile time if instantiated, but otherwise
    //!   have no effect.
-   scoped_lock(mutex_type& m, detail::try_to_lock_type)
+   scoped_lock(mutex_type& m, try_to_lock_type)
       : mp_mutex(&m), m_locked(mp_mutex->try_lock())
    {}
 
-   //!Effects: m.timed_lock(abs_time). 
+   //!Effects: m.timed_lock(abs_time).
    //!Postconditions: mutex() == &m. owns() == the return value of the
    //!   m.timed_lock(abs_time) executed within the constructor.
    //!Notes: The constructor will take ownership of the mutex if it can do
@@ -122,21 +120,15 @@ class scoped_lock
    //!   to thisscoped_lock with no blocking. If the scop scoped_lock does not
    //!   own the mutex, then neither will this scoped_lock. Only a moved
    //!   scoped_lock's will match this signature. An non-moved scoped_lock
-   //!   can be moved with the expression: "move(lock);". This
+   //!   can be moved with the expression: "boost::move(lock);". This
    //!   constructor does not alter the state of the mutex, only potentially
    //!   who owns it.
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   explicit scoped_lock(detail::moved_object<scoped_lock<Mutex> > scop)
-      : mp_mutex(0), m_locked(scop.get().owns())
-   {  mp_mutex = scop.get().release(); }
-   #else
-   explicit scoped_lock(scoped_lock<Mutex> &&scop)
+   scoped_lock(BOOST_RV_REF(scoped_lock) scop)
       : mp_mutex(0), m_locked(scop.owns())
    {  mp_mutex = scop.release(); }
-   #endif
 
    //!Effects: If upgr.owns() then calls unlock_upgradable_and_lock() on the
-   //!   referenced mutex. upgr.release() is called. 
+   //!   referenced mutex. upgr.release() is called.
    //!Postconditions: mutex() == the value upgr.mutex() had before the construction.
    //!   upgr.mutex() == 0. owns() == upgr.owns() before the construction.
    //!   upgr.owns() == false after the construction.
@@ -144,22 +136,12 @@ class scoped_lock
    //!   unlocking upgr. If upgr is unlocked, then this scoped_lock will be
    //!   unlocked as well. Only a moved upgradable_lock's will match this
    //!   signature. An non-moved upgradable_lock can be moved with
-   //!   the expression: "move(lock);" This constructor may block if
+   //!   the expression: "boost::move(lock);" This constructor may block if
    //!   other threads hold a sharable_lock on this mutex (sharable_lock's can
    //!   share ownership with an upgradable_lock).
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   explicit scoped_lock(detail::moved_object<upgradable_lock<Mutex> > upgr)
-      : mp_mutex(0), m_locked(false)
-   {
-      upgradable_lock<mutex_type> &u_lock = upgr.get();
-      if(u_lock.owns()){
-         u_lock.mutex()->unlock_upgradable_and_lock();
-         m_locked = true;
-      }
-      mp_mutex = u_lock.release();
-   }
-   #else
-   explicit scoped_lock(upgradable_lock<Mutex> &&upgr)
+   template<class T>
+   explicit scoped_lock(BOOST_RV_REF(upgradable_lock<T>) upgr
+      , typename ipcdetail::enable_if< ipcdetail::is_same<T, Mutex> >::type * = 0)
       : mp_mutex(0), m_locked(false)
    {
       upgradable_lock<mutex_type> &u_lock = upgr;
@@ -169,41 +151,25 @@ class scoped_lock
       }
       mp_mutex = u_lock.release();
    }
-   #endif
 
    //!Effects: If upgr.owns() then calls try_unlock_upgradable_and_lock() on the
    //!referenced mutex:
    //!   a)if try_unlock_upgradable_and_lock() returns true then mutex() obtains
-   //!      the value from upgr.release() and owns() is set to true. 
+   //!      the value from upgr.release() and owns() is set to true.
    //!   b)if try_unlock_upgradable_and_lock() returns false then upgr is
-   //!      unaffected and this scoped_lock construction as the same effects as  
-   //!      a default construction. 
+   //!      unaffected and this scoped_lock construction as the same effects as
+   //!      a default construction.
    //!   c)Else upgr.owns() is false. mutex() obtains the value from upgr.release()
-   //!      and owns() is set to false 
+   //!      and owns() is set to false
    //!Notes: This construction will not block. It will try to obtain mutex
    //!   ownership from upgr immediately, while changing the lock type from a
    //!   "read lock" to a "write lock". If the "read lock" isn't held in the
    //!   first place, the mutex merely changes type to an unlocked "write lock".
    //!   If the "read lock" is held, then mutex transfer occurs only if it can
-   //!   do so in a non-blocking manner.*/
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   scoped_lock(detail::moved_object<upgradable_lock<Mutex> > upgr
-              ,detail::try_to_lock_type)
-      : mp_mutex(0), m_locked(false)
-   {
-      upgradable_lock<mutex_type> &u_lock = upgr.get();
-      if(u_lock.owns()){
-         if((m_locked = u_lock.mutex()->try_unlock_upgradable_and_lock()) == true){
-            mp_mutex = u_lock.release();
-         }
-      }
-      else{
-         u_lock.release();
-      }
-   }
-   #else
-   scoped_lock(upgradable_lock<Mutex> &&upgr
-              ,detail::try_to_lock_type)
+   //!   do so in a non-blocking manner.
+   template<class T>
+   scoped_lock(BOOST_RV_REF(upgradable_lock<T>) upgr, try_to_lock_type
+         , typename ipcdetail::enable_if< ipcdetail::is_same<T, Mutex> >::type * = 0)
       : mp_mutex(0), m_locked(false)
    {
       upgradable_lock<mutex_type> &u_lock = upgr;
@@ -216,40 +182,24 @@ class scoped_lock
          u_lock.release();
       }
    }
-   #endif
 
    //!Effects: If upgr.owns() then calls timed_unlock_upgradable_and_lock(abs_time)
    //!   on the referenced mutex:
    //!   a)if timed_unlock_upgradable_and_lock(abs_time) returns true then mutex()
-   //!      obtains the value from upgr.release() and owns() is set to true. 
+   //!      obtains the value from upgr.release() and owns() is set to true.
    //!   b)if timed_unlock_upgradable_and_lock(abs_time) returns false then upgr
    //!      is unaffected and this scoped_lock construction as the same effects
    //!      as a default construction.
    //!   c)Else upgr.owns() is false. mutex() obtains the value from upgr.release()
-   //!      and owns() is set to false 
+   //!      and owns() is set to false
    //!Notes: This construction will not block. It will try to obtain mutex ownership
    //!   from upgr immediately, while changing the lock type from a "read lock" to a
    //!   "write lock". If the "read lock" isn't held in the first place, the mutex
    //!   merely changes type to an unlocked "write lock". If the "read lock" is held,
    //!   then mutex transfer occurs only if it can do so in a non-blocking manner.
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   scoped_lock(detail::moved_object<upgradable_lock<Mutex> > upgr
-              ,boost::posix_time::ptime &abs_time)
-      : mp_mutex(0), m_locked(false)
-   {
-      upgradable_lock<mutex_type> &u_lock = upgr.get();
-      if(u_lock.owns()){
-         if((m_locked = u_lock.mutex()->timed_unlock_upgradable_and_lock(abs_time)) == true){
-            mp_mutex = u_lock.release();
-         }
-      }
-      else{
-         u_lock.release();
-      }
-   }
-   #else
-   scoped_lock(upgradable_lock<Mutex> &&upgr
-              ,boost::posix_time::ptime &abs_time)
+   template<class T>
+   scoped_lock(BOOST_RV_REF(upgradable_lock<T>) upgr, boost::posix_time::ptime &abs_time
+               , typename ipcdetail::enable_if< ipcdetail::is_same<T, Mutex> >::type * = 0)
       : mp_mutex(0), m_locked(false)
    {
       upgradable_lock<mutex_type> &u_lock = upgr;
@@ -262,41 +212,25 @@ class scoped_lock
          u_lock.release();
       }
    }
-   #endif
 
    //!Effects: If shar.owns() then calls try_unlock_sharable_and_lock() on the
-   //!referenced mutex. 
+   //!referenced mutex.
    //!   a)if try_unlock_sharable_and_lock() returns true then mutex() obtains
-   //!      the value from shar.release() and owns() is set to true. 
+   //!      the value from shar.release() and owns() is set to true.
    //!   b)if try_unlock_sharable_and_lock() returns false then shar is
    //!      unaffected and this scoped_lock construction has the same
-   //!      effects as a default construction. 
+   //!      effects as a default construction.
    //!   c)Else shar.owns() is false. mutex() obtains the value from
-   //!      shar.release() and owns() is set to false 
+   //!      shar.release() and owns() is set to false
    //!Notes: This construction will not block. It will try to obtain mutex
    //!   ownership from shar immediately, while changing the lock type from a
    //!   "read lock" to a "write lock". If the "read lock" isn't held in the
    //!   first place, the mutex merely changes type to an unlocked "write lock".
    //!   If the "read lock" is held, then mutex transfer occurs only if it can
    //!   do so in a non-blocking manner.
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   scoped_lock(detail::moved_object<sharable_lock<Mutex> > shar
-              ,detail::try_to_lock_type)
-      : mp_mutex(0), m_locked(false)
-   {
-      sharable_lock<mutex_type> &s_lock = shar.get();
-      if(s_lock.owns()){
-         if((m_locked = s_lock.mutex()->try_unlock_sharable_and_lock()) == true){
-            mp_mutex = s_lock.release();
-         }
-      }
-      else{
-         s_lock.release();
-      }
-   }
-   #else
-   scoped_lock(sharable_lock<Mutex> &&shar
-              ,detail::try_to_lock_type)
+   template<class T>
+   scoped_lock(BOOST_RV_REF(sharable_lock<T>) shar, try_to_lock_type
+      , typename ipcdetail::enable_if< ipcdetail::is_same<T, Mutex> >::type * = 0)
       : mp_mutex(0), m_locked(false)
    {
       sharable_lock<mutex_type> &s_lock = shar;
@@ -309,7 +243,6 @@ class scoped_lock
          s_lock.release();
       }
    }
-   #endif
 
    //!Effects: if (owns()) mp_mutex->unlock().
    //!Notes: The destructor behavior ensures that the mutex lock is not leaked.*/
@@ -320,30 +253,19 @@ class scoped_lock
    }
 
    //!Effects: If owns() before the call, then unlock() is called on mutex().
-   //!   *this gets the state of scop and scop gets set to a default constructed state. 
+   //!   *this gets the state of scop and scop gets set to a default constructed state.
    //!Notes: With a recursive mutex it is possible that both this and scop own
    //!   the same mutex before the assignment. In this case, this will own the
    //!   mutex after the assignment (and scop will not), but the mutex's lock
    //!   count will be decremented by one.
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   scoped_lock &operator=(detail::moved_object<scoped_lock<Mutex> > scop)
-   {  
-      if(this->owns())
-         this->unlock();
-      m_locked = scop.get().owns();
-      mp_mutex = scop.get().release();
-      return *this;
-   }
-   #else
-   scoped_lock &operator=(scoped_lock<Mutex> &&scop)
-   {  
+   scoped_lock &operator=(BOOST_RV_REF(scoped_lock) scop)
+   {
       if(this->owns())
          this->unlock();
       m_locked = scop.owns();
       mp_mutex = scop.release();
       return *this;
    }
-   #endif
 
    //!Effects: If mutex() == 0 or if already locked, throws a lock_exception()
    //!   exception. Calls lock() on the referenced mutex.
@@ -359,7 +281,7 @@ class scoped_lock
    }
 
    //!Effects: If mutex() == 0 or if already locked, throws a lock_exception()
-   //!   exception. Calls try_lock() on the referenced mutex. 
+   //!   exception. Calls try_lock() on the referenced mutex.
    //!Postconditions: owns() == the value returned from mutex()->try_lock().
    //!Notes: The scoped_lock changes from a state of not owning the mutex, to
    //!   owning the mutex, but only if blocking was not required. If the
@@ -426,40 +348,22 @@ class scoped_lock
       m_locked = false;
       return mut;
    }
- 
-   //!Effects: Swaps state with moved lock. 
+
+   //!Effects: Swaps state with moved lock.
    //!Throws: Nothing.
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   void swap(detail::moved_object<scoped_lock<mutex_type> > other)
-   {
-      std::swap(mp_mutex, other.get().mp_mutex);
-      std::swap(m_locked, other.get().m_locked);
-   }
-   #else
-   void swap(scoped_lock<mutex_type> &&other)
+   void swap( scoped_lock<mutex_type> &other)
    {
       std::swap(mp_mutex, other.mp_mutex);
       std::swap(m_locked, other.m_locked);
    }
-   #endif
 
-   /// @cond
+   #if !defined(BOOST_INTERPROCESS_DOXYGEN_INVOKED)
    private:
-   mutex_type *mp_mutex; 
+   mutex_type *mp_mutex;
    bool        m_locked;
-   /// @endcond
+   #endif   //#ifndef BOOST_INTERPROCESS_DOXYGEN_INVOKED
 };
 
-/// @cond
-
-//!This class is movable
-template <class M>
-struct is_movable<scoped_lock<M> >
-{
-   enum {   value = true };
-};
-/// @endcond
-
 } // namespace interprocess
 } // namespace boost
 
